<?php

namespace App\Http\Controllers;

use Carbon\Carbon;

use App\Models\Seat;

use App\Models\Reservation;
use Illuminate\Http\Request;

use App\Mail\CancellationMail;
use Illuminate\Support\Facades\DB;

use App\Exports\ReservationsExport;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use Maatwebsite\Excel\Facades\Excel;
use App\Mail\ReservationConfirmation;

class ReservationController extends Controller
{

    public function index(Request $request)
    {

        $reservations = Reservation::all();


        return view('backend.modules.reservation.index', compact('reservations'));
    }

    public function store(Request $request)
    {
        // Retrieve reservation date from the session
        $reservationDate = $request->session()->get('selectedDate', now()->toDateString());

        // Validate incoming data
        $validatedData = $request->validate([
            'seat_id' => 'required|exists:seats,id',
            'fullName' => 'required|string|max:255',
            'contactNumber' => 'required|string|digits:11',
            'emailAddress' => 'required|email',
            'eta' => 'required|date_format:H:i',
            'guests' => 'required|integer|min:1',
            'notes' => 'nullable|string',
            'seat_price' => 'nullable|string',
            'door_charge' => 'nullable|string',
        ]);

        // Replace 'reservationDate' in validated data with the session value
        $validatedData['reservationDate'] = $reservationDate;

        // Use a transaction to ensure atomicity
        DB::transaction(function () use ($validatedData, $reservationDate) { // Include $reservationDate here
            // Create a new reservation
            $reservation = Reservation::create([
                'seat_id' => $validatedData['seat_id'],
                'full_name' => $validatedData['fullName'],
                'contact_number' => $validatedData['contactNumber'],
                'email' => $validatedData['emailAddress'],
                'reservation_date' => $validatedData['reservationDate'], // From session
                'estimated_time_arrival' => $validatedData['eta'],
                'number_of_guests' => $validatedData['guests'],
                'amount_to_pay' => $validatedData['seat_price'],
                'door_charge' => $validatedData['door_charge'],
                'notes' => $validatedData['notes'],
            ]);

            // Send the reservation confirmation email
            $seat_id = $validatedData['seat_id'];
            $fullName = $validatedData['fullName'];
            $contact_number = $validatedData['contactNumber'];
            $email = $validatedData['emailAddress'];
            $reservation_date = $validatedData['reservationDate'];
            $estimated_time_arrival = $validatedData['eta'];
            $number_of_guests = $validatedData['guests'];
            $amount_to_pay = $validatedData['seat_price'];
            $door_charge = $validatedData['door_charge'];
            $notes = $validatedData['notes'];


            // Send email
            Mail::to($validatedData['emailAddress'])
                ->cc('xclv.davao@gmail.com')  // Adding the CC recipient
                ->send(new ReservationConfirmation(
                    $seat_id,
                    $fullName,
                    $contact_number,
                    $email,
                    $reservation_date,
                    $estimated_time_arrival,
                    $number_of_guests,
                    $amount_to_pay,
                    $door_charge,
                    $notes
                ));


            // Send SMS using iTexMo API
            // $response = Http::post('https://api.itexmo.com/api/broadcast-otp', [
            //     'Email' => 'cloud.rweb@gmail.com',
            //     'Password' => 'RW3bS0luti0nsC0rp',
            //     'Recipients' => [$validatedData['contactNumber']], // Use the contact number from the form
            //     'Message' => "Hello {$validatedData['fullName']}, your reservation has been confirmed. We look forward to welcoming you on {$reservationDate} at {$validatedData['eta']}. Total guests: {$validatedData['guests']}. Amount to pay: {$validatedData['amount_to_pay']}. Thank you for choosing us!",
            //     'ApiCode' => 'PR-RWEBS392247_7EMET',
            //     'SenderId' => 'R Web Corp',
            // ]);

            // Check if the response was successful
            // if ($response->successful()) {
            //     // Optionally log or handle success
            // } else {
            //     // Handle failure (logging, etc.)
            // }
        });

        // Set success message and redirect
        return redirect()->back()
            ->with('success', 'Reservation successfully created, email sent, and SMS notification sent!');
    }



    public function export()
    {
        return Excel::download(new ReservationsExport, 'reservations.xlsx');
    }

    public function destroy($id)
    {
        $reservation = Reservation::findOrFail($id);

        // You can also delete the associated seat status update if needed
        Seat::where('id', $reservation->seat_id)->update(['status' => 'available']);

        // Delete the reservation
        $reservation->delete();

        // Redirect back with a success message
        return redirect()->route('admin.reservation')
            ->with('success', 'Reservation successfully deleted!');
    }

    // app/Http/Controllers/ReservationController.php

    public function confirm($id)
    {
        $reservation = Reservation::findOrFail($id);

        // Toggle the confirmation status
        $reservation->is_confirmed = !$reservation->is_confirmed;

        // Set status explicitly to 'confirmed' if is_confirmed is true
        if ($reservation->is_confirmed) {
            $reservation->status = 'confirmed'; // Adjust the status field name and value if needed
        } else {
            $reservation->status = 'pending'; // Or another default value when not confirmed
        }

        $reservation->save();

        // Redirect back with a success message
        return redirect()->route('admin.reservation')
            ->with('success', 'Reservation confirmation status updated successfully!');
    }

    public function cancel($id)
    {
        // Fetch the reservation or fail if not found
        $reservation = Reservation::findOrFail($id);

        // Retrieve the required information
        $email = $reservation->email; // Adjust this field name as per your database structure
        $fullName = $reservation->full_name; // Adjust this field name as per your database structure
        $seat_id = $reservation->seat_id; // Adjust this field name as per your database structure
        $contact_number = $reservation->contact_number; // Adjust this field name as per your database structure
        $reservation_date = $reservation->reservation_date; // Adjust this field name as per your database structure
        $estimated_time_arrival = $reservation->estimated_time_arrival; // Adjust this field name as per your database structure
        $number_of_guests = $reservation->number_of_guests; // Adjust this field name as per your database structure
        $amount_to_pay = $reservation->amount_to_pay; // Adjust this field name as per your database structure
        $door_charge = $reservation->door_charge; // Adjust this field name as per your database structure
        $notes = $reservation->notes; // Adjust this field name as per your database structure

        // Send cancellation email
        Mail::to($email)->send(new CancellationMail(
            $seat_id,
            $fullName,
            $contact_number,
            $email,
            $reservation_date,
            $estimated_time_arrival,
            $number_of_guests,
            $amount_to_pay,
            $door_charge,
            $notes
        ));

        // Set the status to 'cancelled' and save the reservation
        $reservation->status = 'cancelled';
        $reservation->save();

        // Redirect back with a success message
        return redirect()->back() // Ensure the correct route name
            ->with('success', 'Reservation has been cancelled successfully!');
    }
}
